"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useVirtualListItemObserver = void 0;
const lodash_1 = require("lodash");
const react_1 = require("react");
const useVirtualListItemObserver = ({ listRef, items, rowGap, estimateItemInitialHeight, }) => {
    const [itemsHeights, setItemsHeights] = (0, react_1.useState)(items.map(estimateItemInitialHeight));
    const itemsHeightsRef = (0, react_1.useRef)(itemsHeights);
    itemsHeightsRef.current = itemsHeights;
    const observedElements = (0, react_1.useRef)(new WeakMap());
    const oldRaf = (0, react_1.useRef)();
    const resizeObserver = (0, react_1.useMemo)(() => {
        return new ResizeObserver((entries) => {
            if (oldRaf.current) {
                cancelAnimationFrame(oldRaf.current);
                oldRaf.current = undefined;
            }
            oldRaf.current = requestAnimationFrame(() => {
                const sparseModifiedIndexes = [];
                for (const { target, contentRect } of entries) {
                    if (target instanceof HTMLDivElement) {
                        const targetIndex = observedElements.current.get(target);
                        if (targetIndex !== undefined) {
                            sparseModifiedIndexes[targetIndex] = contentRect.height;
                        }
                    }
                }
                const previousHeights = [...itemsHeightsRef.current];
                const newHeights = Object.assign(new Array(), previousHeights, sparseModifiedIndexes);
                if ((0, lodash_1.isEqual)(previousHeights, newHeights)) {
                    oldRaf.current = undefined;
                    return;
                }
                setItemsHeights(newHeights);
                const indexToResetAfter = sparseModifiedIndexes.findIndex((height) => typeof height === 'number');
                if (indexToResetAfter !== -1) {
                    listRef.current?.resetAfterIndex(indexToResetAfter);
                }
                oldRaf.current = undefined;
            });
        });
    }, []);
    const getItemSize = (0, react_1.useCallback)((idx) => {
        const itemHeight = itemsHeights[idx] ?? estimateItemInitialHeight(items[idx]);
        if (rowGap !== undefined && idx !== items.length - 1) {
            return itemHeight + rowGap;
        }
        return itemHeight;
    }, [rowGap, itemsHeights, items, estimateItemInitialHeight]);
    const estimatedItemSize = (0, react_1.useMemo)(() => {
        return (items.reduce((total, _item, index) => total + getItemSize(index), 0) /
            items.length);
    }, [items, getItemSize]);
    const listItemObserver = (0, react_1.useMemo)(() => ({
        observe(element, index) {
            observedElements.current.set(element, index);
            resizeObserver.observe(element);
        },
        unobserve(element) {
            observedElements.current.delete(element);
            resizeObserver.unobserve(element);
        },
    }), [resizeObserver]);
    (0, react_1.useEffect)(() => {
        return () => {
            if (oldRaf.current) {
                cancelAnimationFrame(oldRaf.current);
                oldRaf.current = undefined;
            }
            resizeObserver.disconnect();
        };
    }, []);
    return {
        observer: listItemObserver,
        getItemSize,
        estimatedItemSize,
    };
};
exports.useVirtualListItemObserver = useVirtualListItemObserver;
//# sourceMappingURL=use-virtual-list-item-observer.js.map